package ru.dbotthepony.kstarbound.defs.item

import com.google.gson.GsonBuilder
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.io.KConcreteTypeAdapter
import ru.dbotthepony.kstarbound.registerTypeAdapter
import ru.dbotthepony.kvector.vector.ndouble.Vector2d

data class ItemDefinition(
	/**
	 * Внутреннее имя предмета, как строка
	 */
	val itemName: String,

	/**
	 * Цена в пикселях
	 */
	val price: Long = 0L,

	/**
	 * Редкость как [ItemRarity]
	 */
	val rarity: ItemRarity = ItemRarity.COMMON,

	/**
	 * Категория предмета, определяет, в какую вкладку инвентаря оно попадает
	 */
	val category: String? = null,

	/**
	 * Иконка в инвентаре, относительный и абсолютный пути
	 */
	val inventoryIcon: String? = null,

	/**
	 * Описание предмета
	 */
	val description: String = "...",

	/**
	 * Название предмета
	 */
	val shortdescription: String = "...",

	/**
	 * Теги предмета
	 */
	val itemTags: List<String> = listOf(),

	/**
	 * При подборе предмета мгновенно заставляет игрока изучить эти рецепты крафта
	 */
	val learnBlueprintsOnPickup: List<String> = listOf(),

	/**
	 * Максимальное количество предмета в стопке, по умолчанию 9999
	 */
	val maxStack: Long = 9999L,

	/**
	 * snip
	 */
	val eventCategory: String? = null,

	/**
	 * Заставляет предмет "использовать" сразу же при подборе
	 */
	val consumeOnPickup: Boolean = false,

	/**
	 * Запускает следующие квест(ы) при подборе
	 */
	val pickupQuestTemplates: List<String> = listOf(),

	/**
	 * Используется в костях-ископаемых
	 */
	val race: String? = null,
	val displayImage: String? = null,
	val displayoffset: Vector2d? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val fossilSetName: String? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val setIndex: Int? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val setCount: Int? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val setCollectables: Map<String, String>? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val completeFossilIcon: String? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val completeFossilObject: String? = null,

	/**
	 * Используется в костях-ископаемых
	 */
	val completeSetDescriptions: FossilSetDescription? = null,

	/**
	 * Заставляет SAIL/прочих болтать при подборе предмета в первый раз
	 */
	val radioMessagesOnPickup: List<String> = listOf(),

	/**
	 * Топливо корабля
	 */
	val fuelAmount: Long? = null,
) {
	data class FossilSetDescription(
		val price: Long = 0L,
		val shortdescription: String = "...",
		val description: String = "..."
	)

	companion object {
		val ADAPTER = KConcreteTypeAdapter.Builder(ItemDefinition::class)
			.plain(ItemDefinition::itemName)
			.plain(ItemDefinition::price)
			.plain(ItemDefinition::rarity)
			.plain(ItemDefinition::category)
			.plain(ItemDefinition::inventoryIcon, Starbound::readingFolderTransformerNullable)
			.plain(ItemDefinition::description)
			.plain(ItemDefinition::shortdescription)

			.list(ItemDefinition::itemTags)
			.list(ItemDefinition::learnBlueprintsOnPickup)

			.plain(ItemDefinition::maxStack)
			.plain(ItemDefinition::eventCategory)
			.plain(ItemDefinition::consumeOnPickup)
			.list(ItemDefinition::pickupQuestTemplates)

			.plain(ItemDefinition::race)
			.plain(ItemDefinition::displayImage, Starbound::readingFolderTransformerNullable)
			.plain(ItemDefinition::displayoffset)
			.plain(ItemDefinition::fossilSetName)
			.plain(ItemDefinition::setIndex)
			.plain(ItemDefinition::setCount)
			.map(ItemDefinition::setCollectables, String::class)
			.plain(ItemDefinition::completeFossilIcon)
			.plain(ItemDefinition::completeFossilObject)

			.plain(ItemDefinition::completeSetDescriptions)
			.list(ItemDefinition::radioMessagesOnPickup)
			.plain(ItemDefinition::fuelAmount)

			.build()

		val FOSSIL_ADAPTER = KConcreteTypeAdapter.Builder(FossilSetDescription::class)
			.plain(FossilSetDescription::price)
			.plain(FossilSetDescription::shortdescription)
			.plain(FossilSetDescription::description)
			.build()

		fun registerGson(gsonBuilder: GsonBuilder) {
			gsonBuilder.registerTypeAdapter(ADAPTER)
			gsonBuilder.registerTypeAdapter(FOSSIL_ADAPTER)
		}
	}
}
